import dataclasses as dc
from typing import List
from enum import Enum

from armarx_skills.manager import dto
from armarx_memory.ice_conv.ice_converter import IceConverter
from armarx_skills.manager.fluxio_node import Node, NodeTypes
from armarx_skills.manager.fluxio_parameter import Parameter, FluxioParameterConv


class ControlType(str, Enum):
    SPLITTER = "SPLITTER"
    AND_MERGER = "AND_MERGER"


@dc.dataclass
class ControlNode(Node):

    controlType: ControlType
    parameters: List[Parameter]

    def __init__(
        self,
        nodeType: NodeTypes,
        nodeId: str,
        xPos: float,
        yPos: float,
        controlType: ControlType,
        parameters: List[Parameter],
    ) -> None:
        super().__init__(nodeType, nodeId, xPos, yPos)
        self.controlType = controlType
        self.parameters = parameters


class FluxioControlNodeConv(IceConverter):
    def __init__(self):
        super().__init__()

    @classmethod
    def _import_dto(cls):
        return dto.FluxioNode

    def _from_ice(self, dt: dto.FluxioNode) -> ControlNode:
        return ControlNode(
            nodeType=NodeTypes(dt.nodeType),
            nodeId=dt.nodeId,
            xPos=dt.xPos,
            yPos=dt.yPos,
            controlType=ControlType(dt.controlType),
            parameters=[FluxioParameterConv().from_ice(p) for p in dt.parameters],
        )

    def _to_ice(self, bo: ControlNode) -> dto.FluxioNode:
        return dto.FluxioNode(
            nodeType=bo.nodeType,
            nodeId=bo.nodeId,
            xPos=bo.xPos,
            yPos=bo.yPos,
            controlType=bo.controlType,
            parameters=[FluxioParameterConv().to_ice(p) for p in bo.parameters],
        )
