import dataclasses as dc
from enum import Enum
import numpy as np
from numpy import ndarray

from Ice import Object


class FluxioDataIdentifier(str, Enum):
    STRING = "STRING"
    BOOL = "BOOL"
    INT = "INT"
    LONG = "LONG"
    FLOAT = "FLOAT"
    DOUBLE = "DOUBLE"
    EVENT = "EVENT"
    LIST = "LIST"
    DICT = "DICT"
    NDARRAY = "NDARRAY"


@dc.dataclass
class FluxioBaseData:  # EVENT
    data_identifier: FluxioDataIdentifier

    def __init__(self, data_identifier: FluxioDataIdentifier):
        self.data_identifier = data_identifier


@dc.dataclass
class FluxioSimpleData(
    FluxioBaseData
):  # BOOL=bool, INT=int, LONG=int, FLOAT=float, DOUBLE=flout, STRING=str
    value: str | bool | int | float

    def __init__(
        self, data_identifier: FluxioDataIdentifier, value: str | bool | int | float
    ):
        super().__init__(data_identifier)
        self.value = value


@dc.dataclass
class FluxioListData(FluxioBaseData):
    elements: list[FluxioBaseData]

    def __init__(
        self, data_identifier: FluxioDataIdentifier, elements: list[FluxioBaseData]
    ):
        super().__init__(data_identifier)
        self.elements = elements


@dc.dataclass
class FluxioDictEntry:
    key: str
    value: FluxioBaseData


@dc.dataclass
class FluxioDictData(FluxioBaseData):
    elements: list[FluxioDictEntry]

    def __init__(self, data_identifier: FluxioDataIdentifier):
        super().__init__(data_identifier)
        self.elements = []

    def add_attribute(self, key: str, value: FluxioBaseData):
        self.elements.append(FluxioDictEntry(key=key, value=value))


@dc.dataclass
class FluxioNDArrayData(FluxioBaseData):
    shape: tuple[int, ...]
    base: ndarray

    def __init__(
        self,
        data_identifier: FluxioDataIdentifier,
        shape: tuple[int, ...],
        base: ndarray,
    ):
        super().__init__(data_identifier)
        self.shape = shape
        self.base = base
