import numpy as np
from armarx_memory.aron.aron_ice_types import AronDataIceTypes
from armarx_memory.aron.conversion.ndarray.common import (
    ndarray_from_aron,
    ndarray_to_aron,
)
from armarx_skills.manager.fluxio_data import (
    FluxioBaseData,
    FluxioDataIdentifier,
    FluxioSimpleData,
    FluxioDictData,
    FluxioNDArrayData,
    FluxioListData,
)


# :)
class FluxioDataAronConverter:
    @classmethod
    def aron_data_to_fluxio_data(
        cls, aron_data: AronDataIceTypes.Data
    ) -> FluxioBaseData | FluxioSimpleData | FluxioDictData | FluxioNDArrayData | FluxioListData:
        ret = None
        if isinstance(aron_data, AronDataIceTypes.String):
            ret = FluxioSimpleData(FluxioDataIdentifier.STRING, aron_data.value)
        elif isinstance(aron_data, AronDataIceTypes.Bool):
            ret = FluxioSimpleData(FluxioDataIdentifier.BOOL, aron_data.value)
        elif isinstance(aron_data, AronDataIceTypes.Int):
            ret = FluxioSimpleData(FluxioDataIdentifier.INT, aron_data.value)
        elif isinstance(aron_data, AronDataIceTypes.Long):
            ret = FluxioSimpleData(FluxioDataIdentifier.LONG, aron_data.value)
        elif isinstance(aron_data, AronDataIceTypes.Float):
            ret = FluxioSimpleData(FluxioDataIdentifier.FLOAT, aron_data.value)
        elif isinstance(aron_data, AronDataIceTypes.Double):
            ret = FluxioSimpleData(FluxioDataIdentifier.DOUBLE, aron_data.value)
        elif isinstance(aron_data, AronDataIceTypes.List):
            elements = [cls.aron_data_to_fluxio_data(e) for e in aron_data.elements]
            ret = FluxioListData(FluxioDataIdentifier.LIST, elements)
        elif isinstance(aron_data, AronDataIceTypes.Dict):
            ret = FluxioDictData(FluxioDataIdentifier.DICT)
            for k, v in aron_data.elements.items():
                ret.add_attribute(k, cls.aron_data_to_fluxio_data(v))
        elif isinstance(aron_data, AronDataIceTypes.NDArray):
            numpy_nparray = ndarray_from_aron(aron_data)
            assert numpy_nparray.base is not None
            ret = FluxioNDArrayData(
                FluxioDataIdentifier.NDARRAY, numpy_nparray.shape, numpy_nparray.base
            )
        return ret

    @classmethod
    def fluxio_data_to_aron(
        cls,
        fluxio_data: FluxioBaseData,
    ) -> any:  # just take the any and stop complaining
        ret = None
        identifier = fluxio_data.data_identifier
        if identifier == FluxioDataIdentifier.STRING:
            ret = AronDataIceTypes.String(value=fluxio_data.value)
        elif identifier == FluxioDataIdentifier.BOOL:
            ret = AronDataIceTypes.Bool(value=fluxio_data.value)
        elif identifier == FluxioDataIdentifier.INT:
            ret = AronDataIceTypes.Int(value=fluxio_data.value)
        elif identifier == FluxioDataIdentifier.LONG:
            ret = AronDataIceTypes.Long(value=fluxio_data.value)
        elif identifier == FluxioDataIdentifier.FLOAT:
            ret = AronDataIceTypes.Float(value=fluxio_data.value)
        elif identifier == FluxioDataIdentifier.DOUBLE:
            ret = AronDataIceTypes.Double(value=fluxio_data.value)
        elif identifier == FluxioDataIdentifier.LIST:
            elements = [cls.fluxio_data_to_aron(e) for e in fluxio_data.elements]
            ret = AronDataIceTypes.List(elements=elements)
        elif identifier == FluxioDataIdentifier.DICT:
            elements = {
                e.key: cls.fluxio_data_to_aron(e.value) for e in fluxio_data.elements
            }
            ret = AronDataIceTypes.Dict(elements=elements)
        elif identifier == FluxioDataIdentifier.NDARRAY:
            numpy_ndarray = np.ndarray(
                shape=fluxio_data.shape,
                dtype=fluxio_data.base.dtype,
                buffer=fluxio_data.base,
            )
            ret = ndarray_to_aron(numpy_ndarray)
        return ret
