import dataclasses as dc
from enum import Enum
from typing import List

from armarx_skills.manager import dto
from armarx_memory.ice_conv.ice_converter import IceConverter


class FluxioExceptionType(str, Enum):
    BAD_REQUEST = "BAD_REQUEST"
    UNAUTHORIZED = "UNAUTHORIZED"
    FORBIDDEN = "FORBIDDEN"
    NOT_FOUND = "NOT_FOUND"
    UNKNOWN = "UNKNOWN"


@dc.dataclass
class FluxioException(BaseException):
    """
    A class representing a Fluxio Exception.

    Attributes:
    message (str): The reason for the exception
    type (FluxioExceptionType): The type of the exception
    contextList (List[str]): A list of context strings
    """

    message: str
    type: FluxioExceptionType
    contextList: List[str] = dc.field(default_factory=list)


class FluxioExceptionConv(IceConverter):
    """
    A class representing a FluxioProviderConv object, which is a subclass of IceConverter.

    Attributes:
    - None

    Methods:

    """

    def __init__(self):
        """
        Initializes a new FluxioProviderConv object.

        Parameters:
        - None

        Return:
        - None
        """
        super().__init__()

    def _from_ice(self, dt: dto.FluxioException):
        return FluxioException(dt.message, FluxioExceptionType(dt.type), dt.contextList)

    def _to_ice(self, bo: FluxioException):
        return dto.FluxioException(bo.reason, bo.type, bo.contextList)

    @classmethod
    def _import_dto(cls):
        """
        This method is a class method that returns the FluxioIdentificator data transfer object (DTO).

        Return:
        dto.FluxioIdentificator: The FluxioIdentificator data transfer object.

        Raises:
        None
        """
        return dto.FluxioException
