import dataclasses as dc

from armarx_skills.manager import dto
from armarx_memory.ice_conv.ice_converter import IceConverter


@dc.dataclass
class Provider:
    """
    A class representing a provider for skills.

    Attributes:
    id (str): The unique identifier for the provider.
    name (str): The name of the provider.
    """

    id: str
    name: str


class FluxioProviderConv(IceConverter):
    """
    A class representing a FluxioProviderConv object, which is a subclass of IceConverter.

    Attributes:
    - None

    Methods:
    - __init__: Initializes a new FluxioProviderConv object.
    - _import_dto: Imports the DTO class for the converter.
    - _from_ice: Converts the DTO to the Provider object.
    - _to_ice: Converts the Provider object to the DTO.
    """

    def __init__(self):
        """
        Initializes a new FluxioProviderConv object.

        Parameters:
        - None

        Return:
        - None
        """
        super().__init__()

    @classmethod
    def _import_dto(cls):
        """
        This method is a class method that imports and returns the FluxioProvider data transfer object (DTO).

        Return:
        dto.FluxioProvider: The FluxioProvider data transfer object.

        Raises:
        None
        """
        return dto.FluxioProvider

    def _from_ice(self, dt: dto.FluxioProvider) -> Provider:
        """
        Converts a data transfer object (DTO) representing a FluxioProvider to a Provider object.

        Parameters:
        dt (dto.FluxioProvider): The data transfer object representing a FluxioProvider.

        Return:
        Provider: A Provider object created from the data transfer object.

        Raises:
        None
        """
        return Provider(id=dt.id, name=dt.name)

    def _to_ice(self, bo: Provider) -> dto.FluxioProvider:
        """
        Converts a Provider business object to a FluxioProvider data transfer object.

        Parameters:
        bo (Provider): The Provider business object to convert.

        Return:
        dto.FluxioProvider: The converted FluxioProvider data transfer object.

        Raises:
        None
        """
        return dto.FluxioProvider(id=bo.id, name=bo.name)

