import dataclasses as dc
import typing as ty

from armarx_skills.callback import dti as callback_dti
from armarx_skills.provider import dti as provider_dto
from armarx_skills.provider import dto as provider_dto
from armarx_skills.provider.skill_id import SkillID, SkillIdConv

from armarx_memory.ice_conv.ice_converter import IceConverter
from armarx_memory.aron.conversion import pythonic_from_to_aron_ice


@dc.dataclass
class SkillExecutionRequest:
    skill_id: SkillID
    executor_name: str
    parameters: ty.Dict[str, ty.Any]
    callback_interface: ty.Optional[callback_dti.SkillProviderCallbackInterfacePrx] = None


class SkillExecutionRequestConv(IceConverter):

    def __init__(self):
        super().__init__()
        self.skill_id_conv = SkillIdConv()

    @classmethod
    def _import_dto(cls):
        return provider_dto.SkillExecutionRequest

    def _from_ice(self, dt: provider_dto.SkillExecutionRequest) -> SkillExecutionRequest:
        return SkillExecutionRequest(
            skill_id=self.skill_id_conv.from_ice(dt.skillId),
            executor_name=dt.executorName,
            parameters=pythonic_from_to_aron_ice.pythonic_from_aron_ice(dt.parameters),
            callback_interface=dt.callbackInterface,
        )

    def _to_ice(self, bo: SkillExecutionRequest) -> provider_dto.SkillExecutionRequest:
        return provider_dto.SkillExecutionRequest(
            skillId=self.skill_id_conv.to_ice(bo.skill_id),
            executorName=bo.executor_name,
            parameters=pythonic_from_to_aron_ice.pythonic_to_aron_ice(bo.parameters),
            callbackInterface=bo.callback_interface,
        )
