import Ice

import typing as ty

from armarx_memory.aron.aron_ice_types import AronDataIceTypes

from armarx_skills.provider import dti, dto


class SkillProviderClient:
    """
    Client-side wrapper of the dti.SkillProviderInterface.

    This class can be used to use an existing skill provider, not implement one.

    TODO: Add Pythonic wrappers.
    """

    def __init__(
            self,
            proxy: dti.SkillProviderInterfacePrx = None,
    ):
        self.proxy = proxy

    def get_skill_description_ice(
            self,
            skill_id: dto.SkillID,
    ) -> ty.Union[Ice.Unset, dto.SkillDescription]:
        return self.proxy.getSkillDescription(
            skill=skill_id,
        )

    def get_skill_descriptions_ice(
            self,
    ) -> dto.SkillDescriptionMap:
        return self.proxy.getSkillDescriptions()


    def get_skill_execution_status_ice(
            self,
            execution_id: dto.SkillExecutionID,
    ) -> ty.Union[Ice.Unset, dto.SkillStatusUpdate]:
        return self.proxy.getSkillExecutionStatus(
            executionId=execution_id,
        )

    def get_skill_execution_statuses_ice(
            self,
    ) -> dto.SkillStatusUpdateMap:
        return self.proxy.getSkillExecutionStatuses()

    def execute_skill_ice(
            self,
            request: dto.SkillExecutionRequest,
    ) -> dto.SkillStatusUpdate:
        return self.proxy.executeSkill(
            executionInfo=request,
        )

    def execute_skill_async_ice(
            self,
            request: dto.SkillExecutionRequest,
    ) -> dto.SkillExecutionID:
        return self.proxy.executeSkillAsync(
            executionInfo=request,
        )

    def update_skill_parameters_ice(
            self,
            execution_id: dto.SkillExecutionID,
            parameters: AronDataIceTypes.Dict,
    ) -> dto.ParameterUpdateResult:
        return self.proxy.updateSkillParameters(
            executionId=execution_id,
            params=parameters,
        )

    def abort_skill_ice(
            self,
            execution_id: dto.SkillExecutionID,
    ) -> dto.AbortSkillResult:
        return self.proxy.abortSkill(
            skill=execution_id,
        )

    def abort_skill_async_ice(
            self,
            execution_id: dto.SkillExecutionID,
    ) -> dto.AbortSkillResult:
        return self.proxy.abortSkillAsync(
            skill=execution_id,
        )
